#!/usr/bin/env python
# -*- coding: utf-8 -*-

import aiohttp
import asyncio
import ssl
import terncy


async def foo():
    return "bar"

def event_hander(t, msg):
    print("got event", msg)

async def main():
    t = terncy.Terncy(
        "homeass_nbhQ43",
        "192.168.2.187",
        443,
        "homeassistant_user",
        "0269c4ce3f0440a3ab409433ba7440fc",
    )
    t.register_event_handler(event_hander)
    # await t.check_token_state(3, "0269c4ce3f0440a3ab409433ba7440fc")
    # await t.start()
    await t.discover()


loop = asyncio.get_event_loop()
loop.run_until_complete(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import websocket
import ssl
import http

h1 = http.client.HTTPSConnection(
    "192.168.2.187", context=ssl._create_unverified_context()
)
# h1.request(
# "POST",
# "/v1/tokens:request",
# '{"reqId":"123456","intent":"requestToken","clientId":"homeass_nbhQ43","username":"homeassistant_user","name":"HA User","role":3}'
# )
# r1 = h1.getresponse()
# print(r1.status, r1.reason)


def on_message(ws, message):
    print("%s", message)
    # print('to {"intent":"pong"}')
    ws.send('{"intent":"pong"}')
    print('{"intent":"pong"}')


def on_error(ws, error):
    print(error)


def on_close(ws):
    print("on close")


def on_open(ws):
    print("on open")
    ws.send('{"intent":"ping"}')


# websocket.enableTrace(True)
ws = websocket.WebSocketApp(
    "wss://192.168.2.117/ws/json?clientId=homeass_nbhQ43&username=homeassistant_user&token=09addbafd811485382b03ff97021bcf3",
    on_message=on_message,
    on_error=on_error,
    on_open=on_open,
    on_close=on_close,
)
# ws.connect("wss://192.168.2.121/ws/json?clientId=homeass_nbhQ43&username=homeassistant_user&token=09addbafd811485382b03ff97021bcf3",
# on_message = on_message,
# on_error = on_error,
# on_close = on_close)
# ws.on_open = on_open
ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json

payload = '{"ee":{"entities":[{"reqId":"123456","intent":"requestToken","clientId":\
        "homeass_nbhQ43","username":"homeassistant_user","name":"HA User",\
        "role":3,"entities":[{"aa":"bb"},{"aa":"cc"}]}]}}'


class BasicRequest:
    def __init__(self, reqId, intent):
        self.reqId = reqId
        self.intent = reqId
        self.reqId = reqId


class BasicEntity:
    def __init__(self):
        pass


class DeviceEntity(BasicEntity):
    def __init__(self):
        pass


# class RequestTokenReq:
# def __init__(self,

# print(payload)


def decode_p(d):
    print(d)
    if "entities" in d:
        if type(d["entities"]) == list:
            print("yes, it's list", len(d["entities"]))
        print(d["entities"])
        print("got entities", d["entities"])
        return d["entities"]
    if "reqId" in d:
        return d["reqId"]
    return d


br = BasicRequest("sdfsdfd", "sync")
# print(json.dumps(br.__dict__))
# print(payload)
obj = json.loads(payload, object_hook=decode_p)
print(obj)
# print(obj['ee'])
# json.loads(payload, object_pairs_hook=decode_p)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import asyncio
import logging
import json
import ssl
import uuid
import version
from datetime import datetime
from enum import Enum
from zeroconf import ServiceBrowser, Zeroconf
import aiohttp

logger = logging.getLogger("terncy.log")


def _next_req_id():
    return uuid.uuid4().hex[0:8]


class TokenState(Enum):
    INVALID = -1
    REQUESTED = 1
    APPROVED = 3


class TerncyZCListener(object):
    def __init__(self, terncy):
        self.terncy = terncy

    def remove_service(self, zeroconf, svc_type, name):
        uuid = name.replace("." + svc_type, "")
        if uuid in self.terncy.discovered_homecenters:
            self.terncy.discovered_homecenters.pop(uuid)

    def add_service(self, zeroconf, svc_type, name):
        info = zeroconf.get_service_info(svc_type, name)
        uuid = name.replace("." + svc_type, "")
        txt_records = {"uuid": uuid}
        for k in info.properties:
            txt_records[k.decode("utf-8")] = info.properties[k].decode("utf-8")

        self.terncy.discovered_homecenters[uuid] = txt_records
        print("\nservice state\n", self.terncy.discovered_homecenters)


class Terncy:
    def __init__(self, client_id, ip, port=443, username="", token=""):
        self.client_id = client_id
        self.ip = ip
        self.port = port
        self.username = username
        self.token = token
        self.token_id = -1
        self.token_state = TokenState.INVALID
        self.discovered_homecenters = {}
        self._connection = None
        self._pending_requests = {}
        self._event_handler = None

    def register_event_handler(self, handler):
        self._event_handler = handler

    async def request_token(self, username, name):
        url = "https://%s:%d/v1/tokens:request" % (self.ip, self.port)
        async with aiohttp.ClientSession() as session:
            data = {
                "reqId": _next_req_id(),
                "intent": "requestToken",
                "clientId": self.client_id,
                "username": self.username,
                "name": name,
                "role": 3,
            }
            async with session.post(
                url,
                data=json.dumps(data),
                ssl=ssl._create_unverified_context(),
            ) as response:
                print("status:", response.status)
                print("headers", response.headers)
                body = await response.json()
                print("body:", body)
                state = TokenState.INVALID
                token = ""
                token_id = -1
                if "state" in body:
                    state = body["state"]
                if "id" in body:
                    token_id = body["id"]
                if "token" in body:
                    token = body["token"]
                return response.status, token_id, token, state

    async def delete_token(self, token_id, token):
        url = "https://%s:%d/v1/tokens:request" % (self.ip, self.port)
        async with aiohttp.ClientSession() as session:
            data = {
                "reqId": _next_req_id(),
                "intent": "deleteToken",
                "clientId": self.client_id,
                "id": token_id,
                "token": token,
            }
            async with session.post(
                url,
                data=json.dumps(data),
                ssl=ssl._create_unverified_context(),
            ) as response:
                print("status:", response.status)
                print("headers", response.headers)
                return response.status

    async def check_token_state(self, token_id, token=""):
        url = "https://%s:%d/v1/tokens:query" % (self.ip, self.port)
        async with aiohttp.ClientSession() as session:
            data = {
                "reqId": _next_req_id(),
                "intent": "queryToken",
                "clientId": self.client_id,
                "token": token,
                "id": token_id,
            }
            print("req", json.dumps(data))
            async with session.post(
                url,
                data=json.dumps(data),
                ssl=ssl._create_unverified_context(),
            ) as response:
                print("status:", response.status)
                print("headers", response.headers)
                body = await response.json()
                print("body:", body)
                state = TokenState.INVALID
                if "state" in body:
                    state = body["state"]
                return response.status, state

    async def discover(self):
        zc = Zeroconf()
        listener = TerncyZCListener(self)
        browser = ServiceBrowser(zc, "_websocket._tcp.local.", listener)
        browser.run()
        await asyncio.sleep(15)
        browser.cancel()
        zc.close()
        return self.discovered_homecenters

    async def start(self):
        """Connect to Terncy system and start event monitor."""
        logger.info(
            "Terncy v%s starting connection to %s:%d.",
            version.__version__,
            self.ip,
            self.port,
        )
        asyncio.ensure_future(self._send_routine())
        return await self._start_websocket()

    async def _send_routine(self):
        while True:
            await asyncio.sleep(5)
            # await self.get_entities("device", True)

    async def _start_websocket(self):
        url = "wss://%s:%d/ws/json?clientId=%s&username=%s&token=%s" % (
            self.ip,
            self.port,
            self.client_id,
            self.username,
            self.token,
        )
        async with aiohttp.ClientSession() as session:
            async with session.ws_connect(
                url, ssl=ssl._create_unverified_context()
            ) as ws:
                self._connection = ws
                async for msg in ws:
                    msgObj = msg.json()
                    print("got message:", datetime.now())
                    print(msgObj)
                    if "rspId" in msgObj:
                        rsp_id = msgObj["rspId"]

                        if rsp_id in self._pending_requests:
                            req = self._pending_requests[rsp_id]
                            req["_response"] = msgObj
                            req["event"].set()
                    if "intent" in msgObj and msgObj["intent"] == "event":
                        if self._event_handler:
                            self._event_handler(self, msgObj)

    async def _wait_for_response(self, req_id, req, timeout):
        evt = asyncio.Event()
        request = {
            "req": req,
            "time": datetime.now(),
            "event": evt,
        }

        self._pending_requests[req_id] = request
        aw = asyncio.ensure_future(evt.wait())
        done, pending = await asyncio.wait({aw}, timeout=timeout)
        if aw in done:
            print("wait response ok", datetime.now())
        else:
            print("wait response timeout", datetime.now())
        if req_id in self._pending_requests:
            self._pending_requests.pop(req_id)

    async def get_entities(self, ent_type, wait_result=False, timeout=5):
        req_id = _next_req_id()
        data = {
            "reqId": req_id,
            "intent": "sync",
            "type": ent_type,
        }
        await self._connection.send_str(json.dumps(data))
        if wait_result:
            await self._wait_for_response(req_id, data, timeout)

    async def set_onoff(self, ent_uuid, state, wait_result=False, timeout=5):
        return self.set_attribute(ent_uuid, "on", state, 0, wait_result)

    async def set_attribute(
        self, ent_uuid, attr, attr_val, method, wait_result=False, timeout=5
    ):
        req_id = _next_req_id()
        data = {
            "reqId": req_id,
            "intent": "execute",
            "entities": [
                {
                    "id": ent_uuid,
                    "attributes": [
                        {
                            "attr": attr,
                            "value": attr_val,
                            "method": method,
                        }
                    ],
                }
            ],
        }
        await self._connection.send_str(json.dumps(data))
        if wait_result:
            await self._wait_for_response(req_id, data, timeout)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

from setuptools import setup

assert sys.version >= "3.6", "Requires Python v3.6 or above."

def read(filename):
    with open(filename, 'r') as file_handle:
        return file_handle.read()

with open("terncy/version.py") as f:
    exec(f.read())

    classifiers = [
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]

    setup(
        name="terncy",
        version=__version__,
        author="Ruixiong Wen",
        author_email="rx.wen218@gmail.com",
        url="https://github.com/rxwen/python-terncy/",
        description="A Python library for controlling Terncy home automation system.",
        long_description=read("README.md"),
        long_description_content_type="text/markdown",
        license="MIT",
        classifiers=classifiers,
        packages=["terncy"],
        install_requires=["enum-compat", "future"],
        test_suite="terncy.tests",
        tests_require=[],
    )
